"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const sinon = require("sinon");
const mongo_entity_1 = require("../mongo_entity");
const testing_1 = require("../testing");
const assert = require("power-assert");
const proxyquire = require("proxyquire");
class CutomEntity extends mongo_entity_1.default {
    constructor() {
        super(...arguments);
        this.name = 'customEntity';
    }
}
describe('Testing Helpers', () => {
    describe('withEntity', () => {
        it('cleans up after error has been thrown', () => __awaiter(this, void 0, void 0, function* () {
            let deleteSpy = null;
            try {
                yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                    deleteSpy = sinon.spy(entity, 'deleteMany');
                    throw new Error('Error');
                }));
            }
            catch (ex) { }
            sinon.assert.calledOnce(deleteSpy);
        }));
        it('can create custom entities', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((entity) => __awaiter(this, void 0, void 0, function* () {
                assert.equal(entity.name, 'customEntity');
            }), { entities: [{ type: CutomEntity }] });
        }));
        it('can create multiple entities', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withEntity((one, two) => __awaiter(this, void 0, void 0, function* () {
                assert.equal(one.collection.collectionName, 'One');
                assert.equal(two.collection.collectionName, 'Two');
            }), { entities: [{ name: 'One' }, { name: 'Two' }] });
        }));
    });
    describe('withContext', () => {
        it('throws error when no initContext is specified', () => __awaiter(this, void 0, void 0, function* () {
            let message = '';
            try {
                yield testing_1.withContext((context) => __awaiter(this, void 0, void 0, function* () { }));
            }
            catch (ex) {
                message = ex.message;
            }
            assert.equal(message, 'No initContext provided, please pass as a parameter or use global config');
        }));
        it('initializes context with connection, executes tests and cleans up afterwards', function () {
            return __awaiter(this, void 0, void 0, function* () {
                function initContext(conn) {
                    return {
                        entity: new mongo_entity_1.default(conn, 'col'),
                        other: {}
                    };
                }
                let inContext = null;
                let deleteSpy = null;
                yield testing_1.withContext((context) => __awaiter(this, void 0, void 0, function* () {
                    assert(context.entity);
                    deleteSpy = sinon.spy(context.entity, 'deleteMany');
                    inContext = context;
                }), initContext);
                sinon.assert.calledOnce(deleteSpy);
            });
        });
        it('throws exception but also cleans up', function () {
            return __awaiter(this, void 0, void 0, function* () {
                function initContext(conn) {
                    return {
                        entity: new mongo_entity_1.default(conn, 'col')
                    };
                }
                let inContext = null;
                let deleteSpy = null;
                try {
                    yield testing_1.withContext((context) => __awaiter(this, void 0, void 0, function* () {
                        deleteSpy = sinon.spy(context.entity, 'deleteMany');
                        inContext = context;
                        throw new Error('Error');
                    }), initContext);
                }
                catch (ex) {
                    assert.equal(ex.message, 'Error');
                }
                sinon.assert.calledOnce(deleteSpy);
            });
        });
        it('can run from the globally declared contextFunction', () => __awaiter(this, void 0, void 0, function* () {
            const initContext = sinon
                .stub()
                .returns({});
            testing_1.config({ initContext });
            yield testing_1.withContext(() => { });
            sinon.assert.calledOnce(initContext);
        }));
        it('can run in the disconnected state', () => __awaiter(this, void 0, void 0, function* () {
            const MongoStub = {
                MongoClient: sinon.stub()
            };
            const { withContext, config } = proxyquire('../testing', { 'mongodb': MongoStub });
            const initContext = (connector) => {
                assert(connector.collection());
                return {};
            };
            config({ initContext });
            yield withContext((context) => { }, null, true);
            sinon.assert.notCalled(MongoStub.MongoClient);
        }));
    });
    describe('withServer', () => {
        let server;
        beforeEach(() => {
            server = {
                startTest: sinon.stub(),
                stopTest: sinon.stub(),
                context: [{ dispose: sinon.stub() }]
            };
        });
        it('throws error when no server is specified', () => __awaiter(this, void 0, void 0, function* () {
            let message = '';
            try {
                yield testing_1.withServer((context) => __awaiter(this, void 0, void 0, function* () { }));
            }
            catch (ex) {
                message = ex.message;
            }
            assert.equal(message, 'No server provided, please pass as a parameter or use global config');
        }));
        it('starts a new server when server is not started', () => __awaiter(this, void 0, void 0, function* () {
            yield testing_1.withServer((context) => { }, server);
            sinon.assert.calledOnce(server.startTest);
        }));
        it('cleans up resources even with exception', () => __awaiter(this, void 0, void 0, function* () {
            try {
                yield testing_1.withServer((context) => { throw ''; }, server);
            }
            catch (ex) { }
            sinon.assert.calledOnce(server.context[0].dispose);
        }));
        it('stops server', function () {
            return __awaiter(this, void 0, void 0, function* () {
                yield testing_1.withServer((server) => {
                    testing_1.disposeServer(true);
                    sinon.assert.calledOnce(server.stopTest);
                }, server);
            });
        });
    });
    describe('Mocha helpers', () => {
        describe('itWithContext', () => {
            testing_1.itWithContext('runs', (context) => {
                assert(true);
            });
        });
        describe('itWithEntity', () => {
            testing_1.itWithEntity('runs', (context) => {
                assert(true);
            });
        });
        describe('itWithServer', () => {
            const server = {
                startTest: sinon.stub(),
                context: []
            };
            testing_1.itWithServer('runs', (context) => {
                assert(true);
            }, server);
        });
    });
    describe('disposeDb', () => {
        it('closes connection', sinon.test(function () {
            return __awaiter(this, void 0, void 0, function* () {
                const dbSpy = {
                    dropDatabase: this.spy(),
                    close: this.spy()
                };
                const MongoStub = {
                    MongoClient: {
                        connect: this.stub().returns(dbSpy)
                    }
                };
                const { getDb, disposeDb } = proxyquire('../testing', { 'mongodb': MongoStub });
                yield getDb();
                yield disposeDb(true);
                sinon.assert.calledOnce(dbSpy.dropDatabase);
                sinon.assert.calledOnce(dbSpy.close);
            });
        }));
    });
});
